@@ -5,6 +5,7 @@ from argparse import ArgumentParser, Action
 import builtins
 from collections.abc import MutableMapping, Iterable, Sequence
 import os
+import sys
 import shlex
 
 from xonsh.dirstack import cd, pushd, popd, dirs, _get_cwd
@@ -12,7 +13,7 @@ from xonsh.environ import locate_binary
 from xonsh.foreign_shells import foreign_shell_data
 from xonsh.jobs import jobs, fg, bg, kill_all_jobs
 from xonsh.history import main as history_alias
-from xonsh.platform import ON_ANACONDA, ON_DARWIN, ON_WINDOWS
+from xonsh.platform import ON_ANACONDA, ON_DARWIN, ON_WINDOWS, scandir
 from xonsh.proc import foreground
 from xonsh.replay import main as replay_main
 from xonsh.timings import timeit_alias
@@ -349,7 +350,7 @@ def which(args, stdin=None, stdout=None, stderr=None):
     parser = ArgumentParser('which', description=desc)
     parser.add_argument('args', type=str, nargs='+',
                         help='The executables or aliases to search for')
-    parser.add_argument('-a', action='store_true', dest='all',
+    parser.add_argument('-a','--all', action='store_true', dest='all',
                         help='Show all matches in $PATH and xonsh.aliases')
     parser.add_argument('-s', '--skip-alias', action='store_true',
                         help='Do not search in xonsh.aliases', dest='skip')
@@ -412,9 +413,9 @@ def which(args, stdin=None, stdout=None, stderr=None):
         for abs_name, from_where in matches:
             if ON_WINDOWS:
                 # Use list dir to get correct case for the filename
-                # i.e. windows is case insesitive but case preserving
+                # i.e. windows is case insensitive but case preserving
                 p, f = os.path.split(abs_name)
-                f = next(s for s in os.listdir(p) if s.lower() == f.lower())
+                f = next(s.name for s in scandir(p) if s.name.lower() == f.lower())
                 abs_name = os.path.join(p, f)
                 if builtins.__xonsh_env__.get('FORCE_POSIX_PATHS', False):
                     abs_name.replace(os.sep, os.altsep)
@@ -458,6 +459,26 @@ def vox(args, stdin=None):
     return vox(args, stdin=stdin)
 
 
+def showcmd(args, stdin=None):
+    """usage: showcmd [-h|--help|cmd args]
+
+    Displays the command and arguments as a list of strings that xonsh would
+    run in subprocess mode. This is useful for determining how xonsh evaluates
+    your commands and arguments prior to running these commands.
+
+    optional arguments:
+      -h, --help            show this help message and exit
+
+    example:
+      >>> showcmd echo $USER can't hear "the sea"
+      ['echo', 'I', "can't", 'hear', 'the sea']
+    """
+    if len(args) == 0 or (len(args) == 1 and args[0] in {'-h', '--help'}):
+        print(showcmd.__doc__.rstrip().replace('\n    ', '\n'))
+    else:
+        sys.displayhook(args)
+
+
 def make_default_aliases():
     """Creates a new default aliases dictionary."""
     default_aliases = {
@@ -485,6 +506,7 @@ def make_default_aliases():
         'timeit': timeit_alias,
         'xonfig': xonfig,
         'scp-resume': ['rsync', '--partial', '-h', '--progress', '--rsh=ssh'],
+        'showcmd': showcmd,
         'ipynb': ['jupyter', 'notebook', '--no-browser'],
         'vox': vox,
         'which': which,
@@ -514,6 +536,7 @@ def make_default_aliases():
             default_aliases[alias] = ['cmd', '/c', alias]
         default_aliases['call'] = ['source-cmd']
         default_aliases['source-bat'] = ['source-cmd']
+        default_aliases['clear'] = 'cls'
         # Add aliases specific to the Anaconda python distribution.
         if ON_ANACONDA:
             def source_cmd_keep_prompt(args, stdin=None):
