@@ -6,11 +6,14 @@ import sys
 import time
 import builtins
 
-from xonsh.tools import XonshError, escape_windows_title_string, ON_WINDOWS, \
-    print_exception, HAVE_PYGMENTS
+from xonsh.tools import XonshError, escape_windows_cmd_string, print_exception
+from xonsh.platform import HAS_PYGMENTS, ON_WINDOWS
+from xonsh.codecache import (should_use_cache, code_cache_name,
+                             code_cache_check, get_cache_filename,
+                             update_cache, run_compiled_code)
 from xonsh.completer import Completer
-from xonsh.environ import multiline_prompt, format_prompt
-if HAVE_PYGMENTS:
+from xonsh.environ import multiline_prompt, format_prompt, partial_format_prompt
+if HAS_PYGMENTS:
     from xonsh.pyghooks import XonshStyle
 
 
@@ -108,14 +111,15 @@ class BaseShell(object):
     """The xonsh shell."""
 
     def __init__(self, execer, ctx, **kwargs):
-        super().__init__(**kwargs)
+        super().__init__()
         self.execer = execer
         self.ctx = ctx
-        self.completer = Completer()
+        if kwargs.get('completer', True):
+            self.completer = Completer()
         self.buffer = []
         self.need_more_lines = False
         self.mlprompt = None
-        if HAVE_PYGMENTS:
+        if HAS_PYGMENTS:
             env = builtins.__xonsh_env__
             self.styler = XonshStyle(env.get('XONSH_COLOR_STYLE'))
         else:
@@ -147,7 +151,7 @@ class BaseShell(object):
         tee = Tee() if store_stdout else io.StringIO()
         try:
             ts0 = time.time()
-            self.execer.exec(code, mode='single', glbs=self.ctx)  # no locals
+            run_compiled_code(code, self.ctx, None, 'single')
             ts1 = time.time()
             if hist.last_cmd_rtn is None:
                 hist.last_cmd_rtn = 0  # returncode for success
@@ -175,11 +179,21 @@ class BaseShell(object):
         if self.need_more_lines:
             return None, code
         src = ''.join(self.buffer)
+        _cache = should_use_cache(self.execer, 'single')
+        if _cache:
+            codefname = code_cache_name(src)
+            cachefname = get_cache_filename(codefname, code=True)
+            usecache, code = code_cache_check(cachefname)
+            if usecache:
+                self.reset_buffer()
+                return src, code
         try:
             code = self.execer.compile(src,
                                        mode='single',
                                        glbs=self.ctx,
                                        locs=None)
+            if _cache:
+                update_cache(code, cachefname)
             self.reset_buffer()
         except SyntaxError:
             if line == '\n':
@@ -213,7 +227,7 @@ class BaseShell(object):
             return
         t = format_prompt(t)
         if ON_WINDOWS and 'ANSICON' not in env:
-            t = escape_windows_title_string(t)
+            t = escape_windows_cmd_string(t)
             os.system('title {}'.format(t))
         else:
             os.write(1, "\x1b]2;{0}\x07".format(t).encode())
@@ -232,7 +246,7 @@ class BaseShell(object):
         env = builtins.__xonsh_env__  # pylint: disable=no-member
         p = env.get('PROMPT')
         try:
-            p = format_prompt(p)
+            p = partial_format_prompt(p)
         except Exception:  # pylint: disable=broad-except
             print_exception()
         self.settitle()
