@@ -19,7 +19,7 @@ from xonsh.tools import (subexpr_from_unbalanced, get_sep,
 
 
 RE_DASHF = re.compile(r'-F\s+(\w+)')
-RE_ATTR = re.compile(r'(\S+(\..+)*)\.(\w*)$')
+RE_ATTR = re.compile(r'([^\s\(\)]+(\.[^\s\(\)]+)*)\.(\w*)$')
 RE_WIN_DRIVE = re.compile(r'^([a-zA-Z]):\\')
 
 
@@ -71,9 +71,9 @@ CHARACTERS_NEED_QUOTES = ' `\t\r\n${}*()"\',?&'
 if ON_WINDOWS:
     CHARACTERS_NEED_QUOTES += '%'
 
-COMPLETION_SKIP_TOKENS = {'man', 'sudo', 'time', 'timeit', 'which'}
+COMPLETION_SKIP_TOKENS = {'man', 'sudo', 'time', 'timeit', 'which', 'showcmd'}
 
-BASH_COMPLETE_SCRIPT = """source {filename}
+BASH_COMPLETE_SCRIPT = """source "{filename}"
 COMP_WORDS=({line})
 COMP_LINE={comp_line}
 COMP_POINT=${{#COMP_LINE}}
@@ -192,14 +192,14 @@ class Completer(object):
             rtn = self.cmd_complete(prefix)
         elif cmd in self.bash_complete_funcs:
             # bash completions
-            rtn = self.bash_complete(prefix, line, begidx, endidx)
-            rtn |= self.path_complete(prefix, path_str_start, path_str_end)
-            return self._filter_repeats(rtn), lprefix
+            comps = self.bash_complete(prefix, line, begidx, endidx)
+            pathcomp =  self.path_complete(prefix, path_str_start, path_str_end)
+            return sorted(self._replace_canonical_rep(comps, pathcomp)), lprefix
         elif prefix.startswith('${') or prefix.startswith('@('):
             # python mode explicitly
             return self._python_mode_completions(prefix, ctx,
                                                  prefixlow,
-                                                 startswither)
+                                                 startswither), lprefix
         elif prefix.startswith('-'):
             comps = self._man_completer.option_complete(prefix, cmd)
             return sorted(comps), lprefix
@@ -270,6 +270,20 @@ class Completer(object):
             reps[canon].append(comp)
         return {max(i, key=len) for i in reps.values()}
 
+    def _replace_canonical_rep(self, comps, replacements):
+        """ Replaces any entries in comps with its cononical duplicate
+            from replacements
+        """
+        replacements = {self._canonical_rep(v): v for v in replacements}
+        rtn = set()
+        for comp in comps:
+            canon = self._canonical_rep(comp)
+            if canon in replacements:
+                rtn.add(replacements[canon])
+            else:
+                rtn.add(comp)
+        return rtn
+
     def find_and_complete(self, line, idx, ctx=None):
         """Finds the completions given only the full code line and a current
         cursor position. This represents an easier alternative to the
@@ -378,11 +392,13 @@ class Completer(object):
                 _tail = space
             else:
                 _tail = ''
+            if start != '' and 'r' not in start and backslash in s:
+                start = 'r%s' % start
             s = s + _tail
             if end != '':
                 if "r" not in start.lower():
                     s = s.replace(backslash, double_backslash)
-                elif s.endswith(backslash):
+                if s.endswith(backslash) and not s.endswith(double_backslash):
                     s += backslash
             if end in s:
                 s = s.replace(end, ''.join('\\%s' % i for i in end))
@@ -448,10 +464,10 @@ class Completer(object):
                  builtins.__xonsh_env__.get('BASH_COMPLETIONS', ()))
         for path in paths:
             if path.is_file():
-                sources.append('source ' + str(path))
+                sources.append('source "{}"'.format(path.as_posix()))
             elif path.is_dir():
                 for _file in (x for x in path.glob('*') if x.is_file()):
-                    sources.append('source ' + str(_file))
+                    sources.append('source "{}"'.format(_file.as_posix()))
         return sources
 
     def _load_bash_complete_funcs(self):
@@ -484,6 +500,8 @@ class Completer(object):
         func_files = {}
         for line in out.splitlines():
             parts = line.split()
+            if ON_WINDOWS:
+                parts = [parts[0], ' '.join(parts[2:])]
             func_files[parts[0]] = parts[-1]
         self.bash_complete_files = {
             cmd: func_files[func]
@@ -493,8 +511,8 @@ class Completer(object):
 
     def _source_completions(self, source):
         return subprocess.check_output(
-            ['bash'], input='\n'.join(source), universal_newlines=True,
-            env=builtins.__xonsh_env__.detype(), stderr=subprocess.DEVNULL)
+                ['bash'], input='\n'.join(source), universal_newlines=True,
+                env=builtins.__xonsh_env__.detype(), stderr=subprocess.DEVNULL)
 
     def attr_complete(self, prefix, ctx):
         """Complete attributes of an object."""
@@ -508,33 +526,28 @@ class Completer(object):
         expr = subexpr_from_unbalanced(expr, '{', '}')
         _ctx = None
         try:
-            val = eval(expr, ctx)
+            val = builtins.__xonsh_execer__.eval(expr, ctx)
             _ctx = ctx
         except:  # pylint:disable=bare-except
             try:
-                val = eval(expr, builtins.__dict__)
+                val = builtins.__xonsh_execer__.eval(expr, builtins.__dict__)
                 _ctx = builtins.__dict__
             except:  # pylint:disable=bare-except
                 return attrs  # anything could have gone wrong!
-        _opts = dir(val)
-        # check whether these options actually work (e.g., disallow 7.imag)
-        opts = []
-        for i in _opts:
-            try:
-                eval('{0}.{1}'.format(expr, i), _ctx)
-            except:  # pylint:disable=bare-except
-                continue
-            else:
-                opts.append(i)
         if len(attr) == 0:
-            opts = [o for o in opts if not o.startswith('_')]
+            opts = [o for o in dir(val) if not o.startswith('_')]
         else:
             csc = builtins.__xonsh_env__.get('CASE_SENSITIVE_COMPLETIONS')
             startswither = startswithnorm if csc else startswithlow
             attrlow = attr.lower()
-            opts = [o for o in opts if startswither(o, attr, attrlow)]
+            opts = [o for o in dir(val) if startswither(o, attr, attrlow)]
         prelen = len(prefix)
         for opt in opts:
+            # check whether these options actually work (e.g., disallow 7.imag)
+            try:
+                builtins.__xonsh_execer__.eval('{0}.{1}'.format(expr, opt), _ctx)
+            except:  # pylint:disable=bare-except
+                continue
             a = getattr(val, opt)
             rpl = opt + '(' if callable(a) else opt
             # note that prefix[:prelen-len(attr)] != prefix[:-len(attr)]
