@@ -5,25 +5,24 @@ import sys
 import time
 import signal
 import builtins
-from subprocess import TimeoutExpired
+from subprocess import TimeoutExpired, check_output
 from io import BytesIO
+from collections import deque
 
 from xonsh.tools import ON_WINDOWS
 
+tasks = deque()
 
 if ON_WINDOWS:
-    def _continue(obj):
-        return None
-
-
-    def _kill(obj):
-        return obj.kill()
+    def _continue(job):
+        job['status'] = "running"
 
+    def _kill(job):
+        check_output(['taskkill', '/F', '/T', '/PID', str(job['obj'].pid)])
 
     def ignore_sigtstp():
         pass
 
-
     def _set_pgrp(info):
         pass
 
@@ -33,58 +32,60 @@ if ON_WINDOWS:
         suspended by ctrl-z.
         """
         _clear_dead_jobs()
-        act = builtins.__xonsh_active_job__
-        if act is None:
-            return
-        job = builtins.__xonsh_all_jobs__[act]
-        obj = job['obj']
-        if job['bg']:
+
+        active_task = get_next_task()
+
+        # Return when there are no foreground active task
+        if active_task is None:
             return
+
+        obj = active_task['obj']
+
+        _continue(active_task)
+
         while obj.returncode is None:
             try:
                 obj.wait(0.01)
             except TimeoutExpired:
                 pass
             except KeyboardInterrupt:
-                obj.kill()
-        if obj.poll() is not None:
-            builtins.__xonsh_active_job__ = None
+                _kill(active_task)
+
+        return wait_for_active_job()
 
 else:
-    def _continue(obj):
-        return signal.SIGCONT
+    def _continue(job):
+        _send_signal(job, signal.SIGCONT)
 
+    def _kill(job):
+        _send_signal(job, signal.SIGKILL)
 
-    def _kill(obj):
-        os.kill(obj.pid, signal.SIGKILL)
-
+    def _send_signal(job, signal):
+        os.killpg(job['pgrp'], signal)
 
     def ignore_sigtstp():
         signal.signal(signal.SIGTSTP, signal.SIG_IGN)
 
-
     def _set_pgrp(info):
         try:
             info['pgrp'] = os.getpgid(info['obj'].pid)
         except ProcessLookupError:
             pass
 
-
     _shell_pgrp = os.getpgrp()
 
     _block_when_giving = (signal.SIGTTOU, signal.SIGTTIN, signal.SIGTSTP)
 
-
     def _give_terminal_to(pgid):
         # over-simplified version of:
         #    give_terminal_to from bash 4.3 source, jobs.c, line 4030
         # this will give the terminal to the process group pgid
         if _shell_tty is not None and os.isatty(_shell_tty):
-            oldmask = signal.pthread_sigmask(signal.SIG_BLOCK, _block_when_giving)
+            oldmask = signal.pthread_sigmask(signal.SIG_BLOCK,
+                                             _block_when_giving)
             os.tcsetpgrp(_shell_tty, pgid)
             signal.pthread_sigmask(signal.SIG_SETMASK, oldmask)
 
-
     # check for shell tty
     try:
         _shell_tty = sys.stderr.fileno()
@@ -94,43 +95,31 @@ else:
     except OSError:
         _shell_tty = None
 
-
-    def wait_for_active_job(signal_to_send=None):
+    def wait_for_active_job():
         """
         Wait for the active job to finish, to be killed by SIGINT, or to be
         suspended by ctrl-z.
         """
         _clear_dead_jobs()
-        act = builtins.__xonsh_active_job__
-        if act is None:
-            return
-        job = builtins.__xonsh_all_jobs__[act]
-        obj = job['obj']
-        if job['bg'] and job['status'] == 'running':
-            return
-        pgrp = job['pgrp']
 
+        active_task = get_next_task()
+
+        # Return when there are no foreground active task
+        if active_task is None:
+            _give_terminal_to(_shell_pgrp)  # give terminal back to the shell
+            return
+
+        pgrp = active_task['pgrp']
+        obj = active_task['obj']
         # give the terminal over to the fg process
         _give_terminal_to(pgrp)
-        # if necessary, send the specified signal to this process
-        # (this hook  was added because vim, emacs, etc, seem to need to have
-        # the terminal when they receive SIGCONT from the "fg" command)
-        if signal_to_send is not None:
-            if signal_to_send == signal.SIGCONT:
-                job['status'] = 'running'
 
-            os.kill(obj.pid, signal_to_send)
-
-            if job['bg']:
-                _give_terminal_to(_shell_pgrp)
-                return
+        _continue(active_task)
 
         _, wcode = os.waitpid(obj.pid, os.WUNTRACED)
         if os.WIFSTOPPED(wcode):
-            job['bg'] = True
-            job['status'] = 'stopped'
             print()  # get a newline because ^Z will have been printed
-            print_one_job(act)
+            active_task['status'] = "stopped"
         elif os.WIFSIGNALED(wcode):
             print()  # get a newline because ^C will have been printed
             obj.signal = (os.WTERMSIG(wcode), os.WCOREDUMP(wcode))
@@ -139,32 +128,37 @@ else:
             obj.returncode = os.WEXITSTATUS(wcode)
             obj.signal = None
 
-        if obj.poll() is not None:
-            builtins.__xonsh_active_job__ = None
+        return wait_for_active_job()
 
-        _give_terminal_to(_shell_pgrp)  # give terminal back to the shell
+
+def get_next_task():
+    """ Get the next active task and put it on top of the queue"""
+    selected_task = None
+    for tid in tasks:
+        task = get_task(tid)
+        if not task['bg'] and task['status'] == "running":
+            selected_task = tid
+            break
+    if selected_task is None:
+        return
+    tasks.remove(selected_task)
+    tasks.appendleft(selected_task)
+    return get_task(selected_task)
+
+
+def get_task(tid):
+    return builtins.__xonsh_all_jobs__[tid]
 
 
 def _clear_dead_jobs():
     to_remove = set()
-    for num, job in builtins.__xonsh_all_jobs__.items():
-        obj = job['obj']
+    for tid in tasks:
+        obj = get_task(tid)['obj']
         if obj.poll() is not None:
-            to_remove.add(num)
-    for i in to_remove:
-        del builtins.__xonsh_all_jobs__[i]
-        if builtins.__xonsh_active_job__ == i:
-            builtins.__xonsh_active_job__ = None
-    if builtins.__xonsh_active_job__ is None:
-        _reactivate_job()
-
-
-def _reactivate_job():
-    if len(builtins.__xonsh_all_jobs__) == 0:
-        return
-    builtins.__xonsh_active_job__ = max(builtins.__xonsh_all_jobs__.items(),
-                                        key=lambda x: x[1]['started'])[0]
-
+            to_remove.add(tid)
+    for job in to_remove:
+        tasks.remove(job)
+        del builtins.__xonsh_all_jobs__[job]
 
 
 def print_one_job(num):
@@ -173,13 +167,12 @@ def print_one_job(num):
         job = builtins.__xonsh_all_jobs__[num]
     except KeyError:
         return
-    act = '*' if num == builtins.__xonsh_active_job__ else ' '
     status = job['status']
     cmd = [' '.join(i) if isinstance(i, list) else i for i in job['cmds']]
     cmd = ' '.join(cmd)
     pid = job['pids'][-1]
     bg = ' &' if job['bg'] else ''
-    print('{}[{}] {}: {}{} ({})'.format(act, num, status, cmd, bg, pid))
+    print('[{}] {}: {}{} ({})'.format(num, status, cmd, bg, pid))
 
 
 def get_next_job_number():
@@ -196,27 +189,23 @@ def add_job(info):
     """
     Add a new job to the jobs dictionary.
     """
-    info['started'] = time.time()
-    info['status'] = 'running'
-    _set_pgrp(info)
     num = get_next_job_number()
+    info['started'] = time.time()
+    info['status'] = "running"
+    _set_pgrp(info)
+    tasks.appendleft(num)
     builtins.__xonsh_all_jobs__[num] = info
-    builtins.__xonsh_active_job__ = num
     if info['bg']:
         print_one_job(num)
 
 
-def _default_sigint_handler(num, frame):
-    raise KeyboardInterrupt
-
-
 def kill_all_jobs():
     """
     Send SIGKILL to all child processes (called when exiting xonsh).
     """
     _clear_dead_jobs()
     for job in builtins.__xonsh_all_jobs__.values():
-        _kill(job['obj'])
+        _kill(job)
 
 
 def jobs(args, stdin=None):
@@ -226,7 +215,7 @@ def jobs(args, stdin=None):
     Display a list of all current jobs.
     """
     _clear_dead_jobs()
-    for j in sorted(builtins.__xonsh_all_jobs__):
+    for j in tasks:
         print_one_job(j)
     return None, None
 
@@ -238,12 +227,13 @@ def fg(args, stdin=None):
     Bring the currently active job to the foreground, or, if a single number is
     given as an argument, bring that job to the foreground.
     """
+
     _clear_dead_jobs()
+    if len(tasks) == 0:
+        return '', 'Cannot bring nonexistent job to foreground.\n'
+
     if len(args) == 0:
-        # start active job in foreground
-        act = builtins.__xonsh_active_job__
-        if act is None:
-            return '', 'Cannot bring nonexistent job to foreground.\n'
+        act = tasks[0]  # take the last manipulated task by default
     elif len(args) == 1:
         try:
             act = int(args[0])
@@ -253,12 +243,15 @@ def fg(args, stdin=None):
             return '', 'Invalid job: {}\n'.format(args[0])
     else:
         return '', 'fg expects 0 or 1 arguments, not {}\n'.format(len(args))
-    builtins.__xonsh_active_job__ = act
-    job = builtins.__xonsh_all_jobs__[act]
+
+    # Put this one on top of the queue
+    tasks.remove(act)
+    tasks.appendleft(act)
+
+    job = get_task(act)
     job['bg'] = False
-    job['status'] = 'running'
+    job['status'] = "running"
     print_one_job(act)
-    wait_for_active_job(_continue(job['obj']))
 
 
 def bg(args, stdin=None):
@@ -268,24 +261,10 @@ def bg(args, stdin=None):
     Resume execution of the currently active job in the background, or, if a
     single number is given as an argument, resume that job in the background.
     """
-    _clear_dead_jobs()
-    if len(args) == 0:
-        # start active job in foreground
-        act = builtins.__xonsh_active_job__
-        if act is None:
-            return '', 'Cannot send nonexistent job to background.\n'
-    elif len(args) == 1:
-        try:
-            act = int(args[0])
-        except ValueError:
-            return '', 'Invalid job: {}\n'.format(args[0])
-        if act not in builtins.__xonsh_all_jobs__:
-            return '', 'Invalid job: {}\n'.format(args[0])
+    res = fg(args, stdin)
+    if res is None:
+        curTask = get_task(tasks[0])
+        curTask['bg'] = True
+        _continue(curTask)
     else:
-        return '', 'bg expects 0 or 1 arguments, not {}\n'.format(len(args))
-    builtins.__xonsh_active_job__ = act
-    job = builtins.__xonsh_all_jobs__[act]
-    job['bg'] = True
-    # When the SIGCONT is sent job['status'] is set to running.
-    print_one_job(act)
-    wait_for_active_job(_continue(job['obj']))
+        return res
